<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\About;
use App\Models\AppUser;
use App\Models\Company;
use App\Models\Testimonials;
use Illuminate\Support\Facades\Validator;
use App\Models\ContactUsHistory;
use App\Models\GroupSubscription;
use App\Models\SpotlightMember;
use Illuminate\Validation\Rule;
use Mail;
use DB;
use Illuminate\Support\Facades\Log;

class CommonController extends Controller
{
    /**
     * @OA\Post(
     *     path="/api/contact-us",
     *     operationId="contactUs",
     *     summary="Contact Us",
     *     description="Contact Us",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"email", "name", "phone_number", "message"},
     *                 @OA\Property(property="email", type="email"),
     *                 @OA\Property(property="name", type="staring"),
     *                 @OA\Property(property="phone_number", type="number"),
     *                 @OA\Property(property="message", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Send mail successfully.",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function contactUs(Request $request)
    {
       try {

            $validator = Validator::make($request->all(), [
                'email' => 'required|string|email',
                'name' => 'required|string',
                'phone_number' => 'required',
                'message' => 'required|string'
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $data = $request->all();

            ContactUsHistory::insert([
                'email' => $data['email'],
                'name' => $data['name'],
                'phone_number' => $data['phone_number'],
                'message' => $data['message']
            ]);

            Mail::send('mails.contact-us', ['data' => $data], function ($message) {
                $message->to(env('ADMIN_EMAIL'), env('ADMIN_NAME'))->subject('Luxury Council - Contact Us');
            });

            try {
                Mail::send('mails.contact-us-thank-you', ['data' => $data], function ($message) use ($data) {
                    $message->to($data['email'], $data['name'])->subject('Thank You for contact us');
                });
            } catch (\Exception $e) {
                Log::error('This is an error message.' . $e);
            }


            $response = [
                'data' => (object)[],
                'status' => true,
                'message' => 'Request has been sent successfully.'
            ];

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => 'Error in sending email; contact administrator for help.'
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Get(
     *     path="/api/about-us",
     *     operationId="aboutUs",
     *     summary="About Us",
     *     description="About Us",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\Response(
     *         response=200,
     *         description="About Us Detail",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function aboutUs(Request $request)
    {
        try {

            $aboutUs = About::select('header', 'header_short_desc', 'header_long_desc', 'image', 'about_id')->first()->toArray();
            $testimonials = Testimonials::select('testimonials_name', 'testimonials_title', 'testimonials_desc')->where('about_id', $aboutUs['about_id'])->get()->toArray();
            $aboutUs['image'] = url('/') . '/aboutus/' . $aboutUs['image'];

            $aboutUs['testimonials'] = [];
            if (!empty($testimonials)) {
                $aboutUs['testimonials'] = $testimonials;
            }

            if (!empty($aboutUs)) {
                $response = [
                    'data' => $aboutUs,
                    'status' => true,
                    'message' => __('messages.detail', ['name' => 'About Us'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

     /**
     * @OA\Post(
     *     path="/api/create-group-subscription",
     *     operationId="createGroupSubscription",
     *     summary="Create Group Subscription",
     *     description="Create Group Subscription",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"company_name", "contact_name", "title", "email", "country_code", "phone_no", "number_of_subscription", "subscription_type", "subscription_start_date"},
     *                 @OA\Property(property="company_name", type="staring"),
     *                 @OA\Property(property="contact_name", type="staring"),
     *                 @OA\Property(property="title", type="staring"),
     *                 @OA\Property(property="email", type="email"),
     *                 @OA\Property(property="country_code", type="staring"),
     *                 @OA\Property(property="phone_no", type="number"),
     *                 @OA\Property(property="number_of_subscription", type="number")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Send mail successfully.",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function createGroupSubscription(Request $request)
    {
       try {
            $validator = Validator::make($request->all(), [
                'company_name' => 'required|string',
                'contact_name' => 'required|string',
                'title' => 'required|string',
                'email' => ['required', 'email' , Rule::unique('group_subscriptions')->whereNull('deleted_at')],
                'country_code' => 'required',
                'phone_no' => ['required', 'numeric', 'digits:10' , Rule::unique('group_subscriptions')->whereNull('deleted_at')],
                'number_of_subscription' => 'required|numeric'
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $data = $request->all();

            $existsCompany = Company::whereRaw('LOWER(`company_name`) = ? ', [trim(strtolower($data['company_name']))])->first();
            if (is_null($existsCompany)) {
                $company = Company::create([
                    'company_name' => $data['company_name'],
                    'company_status' => 1
                ]);
                $companyId = $company->company_id;
            } else {
                $companyId = $existsCompany->company_id;
            }

            GroupSubscription::insert([
                'company_name' => $companyId,
                'contact_name' => $data['contact_name'],
                'title' => $data['title'],
                'email' => $data['email'],
                'country_code' => $data['country_code'],
                'phone_no' => $data['phone_no'],
                'number_of_subscription' => $data['number_of_subscription'],
                'group_subscription_status' => 0,
            ]);
            
            
            $country = \App\Models\Country::select(DB::raw('CONCAT(short_name, " ", "(+", code, ")") AS short_code'), 'short_name')->where('short_name', $data['country_code'])->first();
            
            $short_code = '';
            if(isset($country) && $country->short_code != ''){
                $short_code = $country->short_code;
            }

            $detail = [
                'no_of_user' => $data['number_of_subscription'],
                'company_name' => $data['company_name'],
                'contact' => $data['contact_name'],
                'title' => $data['title'],
                'email' => $data['email'],
                'phone' => $short_code . ' ' . $data['phone_no'],
                'no_of_user' => $data['number_of_subscription'],
                'app_user_name' => \Auth::user()->first_name . ' '. \Auth::user()->last_name,
            ];

            try {
                Mail::send('mails.group-subscription-request', ['detail' => $detail], function($message) use ($data){
                    $message->to($data['email'], env('ADMIN_NAME'))->subject
                       ('Group Subscription Request');
                 });
                
            } catch (\Exception $e) {
                Log::error('This is an error message.'. $e);
            }

            $response = [
                'data' => (object)[],
                'status' => true,
                'message' => __('messages.add', ['name' => 'Group Subscription'])
            ];

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }


     /**
     * @OA\Post(
     *     path="/api/spotlight-member-list",
     *     operationId="spotlightMemberList",
     *     summary="Spotlight Member List",
     *     description="Spotlight Member List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="staring"),
     *                 @OA\Property(property="limit", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Event List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function spotlightMemberList(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;

            $spotlightMember = SpotlightMember::select('spotlight_member_id', 'spotlight_member_company_logo', 'spotlight_member_company_short_description', 'spotlight_member_contact_email', 'spotlight_member_contact_name', 'spotlight_member_status')
                ->status(1)->get();

            $data = $this->paginate($spotlightMember, $limit, $page);

            if (!empty($spotlightMember)) {
                $response = [
                    'data' =>  ['items' => (array) $data->items(), 'total_page' => $data->lastPage()],
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Spotlight Member'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

     /**
     * @OA\Post(
     *     path="/api/spotlight-member-detail",
     *     operationId="spotlightMemberDetail",
     *     summary="Spotlight Member Detail",
     *     description="Spotlight Member Detail",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"spotlight_member_id"},
     *                 @OA\Property(property="spotlight_member_id", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Spotlight Member Detail",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function spotlightMemberDetail(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                'spotlight_member_id' => [
                    'required',
                    Rule::exists('spotlight_members')->where(function ($query) {
                        $query->whereNull('deleted_at');
                    }),
                ],
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $spotlightMemberId = $request->get('spotlight_member_id');

            $spotlightMember = SpotlightMember::select('spotlight_member_id', 'spotlight_member_company_logo', 'spotlight_member_company_short_description', 'spotlight_member_contact_email', 'spotlight_member_contact_name', 'spotlight_member_status', 'spotlight_member_company_long_description')
                ->where('spotlight_member_id', $spotlightMemberId)->first()->toArray();


            if (!empty($spotlightMember)) {
                $response = [
                    'data' => $spotlightMember,
                    'status' => true,
                    'message' => __('messages.detail', ['name' => 'Spotlight Member'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }


            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/chat-user-list",
     *     operationId="chatUserList",
     *     summary="Chat User List",
     *     description="Chat User List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *      @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={ "subscription_id"},
     *                 @OA\Property(property="subscription_id", type="number"),
     *                 @OA\Property(property="page", type="number"),
     *                 @OA\Property(property="limit", type="number"),
     *                 @OA\Property(property="search_text", type="string")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Chat User List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function chatUserList(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'subscription_id' => 'required|in:3,4,5,6'
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $subscriptionId = $request->get('subscription_id');
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $searchText = $request->input('search_text') ?? '';
            $offset = $this->offset($page, $limit);

            $userId = \Auth::guard('api')->user()->app_user_id;

            $chatUser = config('constant.chat_user');

            $sender = \App\Models\Friend::where('receiver_id', $userId)->pluck('sender_id');

            $query = \App\Models\AppUser::select('app_user_id', 'first_name', 'last_name', 'email', 'subscription_id', 'profile_img')
            ->where(function ($q) use ($chatUser, $subscriptionId, $sender) {
                $q->whereIn('subscription_id', $chatUser[$subscriptionId]);
                $q->orWhereIn('app_users.app_user_id', $sender);
            })->where('app_user_id', '!=', $userId)->orderBy('first_name', 'ASC');

            if($searchText != ''){
                $searchText = '%' . $searchText . '%';
                $query->whereRaw("CONCAT(first_name, ' ', last_name) LIKE  '$searchText'");
            }

            $appUser = $query->get();
            $appUser = $this->paginate($appUser, $limit, $page);

            if (!empty($appUser)) {
                $response = [
                    'data' =>  ['items' => (array) $appUser->items(), 'total_page' => $appUser->lastPage(),'total' => $appUser->total()],
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Chat User'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }


    /**
     * @OA\Post(
     *     path="/api/add-friend",
     *     operationId="addFriend",
     *     summary="Add Friend",
     *     description="Add Friend",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"sender_id", "receiver_id"},
     *                 @OA\Property(property="sender_id", type="number"),
     *                 @OA\Property(property="receiver_id", type="number")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Add friend.",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function addFriend(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'sender_id' => 'required|numeric',
                'receiver_id' => 'required|numeric'
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $data = $request->all();

            \App\Models\Friend::firstOrCreate([
                    'sender_id' => $data['sender_id'],
                    'receiver_id' => $data['receiver_id']
                ]);

            $response = [
                'data' => (object)[],
                'status' => true,
                'message' => __('messages.add', ['name' => 'Friend'])
            ];

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

     /**
     * @OA\Post(
     *     path="/api/send-spotlight-mail",
     *     operationId="sendSpotLightMail",
     *     summary="Send Spot Light Mail",
     *     description="Send Spot Light Mail",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"email", "content"},
     *                 @OA\Property(property="email", type="string"),
     *                 @OA\Property(property="content", type="string")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Send Spot Light Mail",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function sendSpotLightMail(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'email' => 'required',
                'content' => 'required'
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $data = $request->all();

            
             
            try {
                Mail::send('mails.send-spot-light-mail', ['data' => $data], function($message) use ($data){
                    $message->to($data['email'], 'Luxury Council–Spotlight Member')->subject
                       ('Luxury Council–Spotlight Member');
                 });
                    
            } catch (\Exception $e) {
                Log::error('This is an error message.'. $e);
            }
             
            


            $response = [
                'data' => (object)[],
                'status' => true,
                'message' => 'Mail has been sent successfully.'
            ];

            return response()->json($response);
        } catch (\Exception $e) {
            Log::error('This is an error message.' . $e);
            
            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => 'Error in sending email; contact administrator for help.'
            ];
            return response()->json($response);
        }
    }



    /**
     * @OA\Post(
     *     path="/api/get-profile-image",
     *     operationId="getProfileImage",
     *     summary="Get Profile Image",
     *     description="Get Profile Image",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"app_user_id"},
     *                 @OA\Property(property="app_user_id", type="string"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Get Profile Image",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function getProfileImage(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'app_user_id' => 'required'
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $data = $request->all();

            $appUserId =  explode(',', $request->app_user_id);
            $appUser = AppUser::select('app_user_id', 'profile_img')->whereIn('app_user_id',  $appUserId)->get();

            $response = [
                'data' => $appUser,
                'status' => true,
                'message' => 'Profile Images.'
            ];

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => 'Something went to wrong. Try after sometime.'
            ];
            return response()->json($response);
        }
    }
}
