<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Models\Event;
use Illuminate\Validation\Rule;

class EventController extends Controller
{

     /**
     * @OA\Post(
     *     path="/api/event-list",
     *     operationId="eventList",
     *     summary="Event List",
     *     description="Event List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="staring"),
     *                 @OA\Property(property="limit", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Event List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function eventList(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $offset = $this->offset($page, $limit);

            $event = Event::select('event_id', 'event_name', 'event_date', 'event_detail', 'hash_tags', 'event_status')->status(1)->skip($offset)->take($limit)->get()->toArray();

            if (!empty($event)) {
                $response = [
                    'data' => $event,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Event'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }


     /**
     * @OA\Post(
     *     path="/api/event-detail",
     *     operationId="eventDetail",
     *     summary="Event Detail",
     *     description="Event Detail",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"event_id"},
     *                 @OA\Property(property="event_id", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Event Detail",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function eventDetail(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                'event_id' => [
                    'required',
                    Rule::exists('events')->where(function ($query) {
                        $query->whereNull('deleted_at');
                    }),
                ],
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $eventId = $request->get('event_id');

            $event = Event::select('event_id', 'event_name', 'event_date', 'event_detail', 'hash_tags', 'event_status')
                ->where('event_id', $eventId)->first()->toArray();


            if (!empty($event)) {
                $response = [
                    'data' => $event,
                    'status' => true,
                    'message' => __('messages.detail', ['name' => 'Event'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }


            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }


     /**
     * @OA\Post(
     *     path="/api/event-mail-history",
     *     operationId="eventMailHistory",
     *     summary="Event Mail History  Detail",
     *     description="Event Mail History  Detail",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="staring"),
     *                 @OA\Property(property="limit", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Event Mail History List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function eventMailHistory(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $offset = $this->offset($page, $limit);

            $authEmail = \Auth::guard('api')->user()->email;

            $event = \App\Models\EventMailLog::leftJoin('events', 'events.event_id', '=', 'event_mail_log.event_id')
                ->select('events.event_id', 'events.event_name', 'events.event_date', 'events.event_detail', 'events.hash_tags', 'events.event_status')
                ->whereRaw("JSON_CONTAINS(event_mail_log.emails, '\"$authEmail\"')")
                ->skip($offset)->take($limit)->get()->toArray();

            if (!empty($event)) {

                $response = [
                    'data' => $event,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Event Mail History'])
                ];

            } else {

                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];

            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }
}
