<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\PaymentLog;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Validator;
use net\authorize\api\contract\v1 as AnetAPI;
use net\authorize\api\controller as AnetController;
use Mail;
use DateTime;
use Illuminate\Support\Facades\Log;

class TransactionController extends Controller
{

    /**
     * @OA\Post(
     *     path="/api/payment",
     *     operationId="payment",
     *     summary="Payment",
     *     description="Payment",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *      @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={ "card_number", "exp_date", "code", "amount", "subscription_id", "subscription_type"},
     *                 @OA\Property(property="card_number", type="number"),
     *                 @OA\Property(property="exp_date", type="number"),
     *                 @OA\Property(property="code", type="number"),
     *                 @OA\Property(property="amount", type="number"),
     *                 @OA\Property(property="subscription_id", type="number"),
     *                 @OA\Property(property="subscription_type", type="number")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Payment",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function payment(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'card_number' => 'required|numeric',
            'exp_date' => 'required',
            'code' => 'required|numeric',
            'amount' => 'required',
            'subscription_id' => 'required',
            'subscription_type' => 'required|in:1,2',
        ]);

        if ($validator->fails()) {
            $validation = [
                'data' => (object) [],
                'status' => false,
                'message' =>  $validator->errors()->first()
            ];
            return response()->json($validation);
        }

        $data = $request->all();
        $cardNumber = $data['card_number'];
        $expDate = $data['exp_date'];
        $code = $data['code'];
        $amount = $data['amount'];
        $subscriptionId = $data['subscription_id'];
        $subscriptionType = $data['subscription_type'];
        $userId = \Auth::guard('api')->user()->app_user_id;
        $type =  $subscriptionType == 1 ? '+1 months' : '+1 year';
        $startDate = date('Y-m-d H:i:s');
        $endDate = date('Y-m-d H:i:s', strtotime($type));

        $uniqueId = uniqid();
        $prefix = "INV";
        $invoiceNumber = $prefix . "-" . $uniqueId;

        /* Create a merchantAuthenticationType object with authentication details retrieved from the constants file */
        $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
        $merchantAuthentication->setName(env('MERCHANT_LOGIN_ID'));
        $merchantAuthentication->setTransactionKey(env('MERCHANT_TRANSACTION_KEY'));

        // dd(env('MERCHANT_LOGIN_ID'), env('MERCHANT_TRANSACTION_KEY'),env('PAYMENT_ENVIRONMENT'));
        // Set the transaction's refId
        $refId = 'ref' . time();
        $intervalLength = 30;

        // Subscription Type Info
        $subscription = new AnetAPI\ARBSubscriptionType();
        $subscription->setName("Subscription");

        $interval = new AnetAPI\PaymentScheduleType\IntervalAType();
        $interval->setLength($intervalLength);
        $interval->setUnit("days");

        $paymentSchedule = new AnetAPI\PaymentScheduleType();
        $paymentSchedule->setInterval($interval);
        $paymentSchedule->setStartDate(new DateTime());
        $paymentSchedule->setTotalOccurrences(9999);

        $subscription->setPaymentSchedule($paymentSchedule);
        $subscription->setAmount($amount);

        $creditCard = new AnetAPI\CreditCardType();
        $creditCard->setCardNumber($cardNumber);
        $creditCard->setExpirationDate($expDate);

        $payment = new AnetAPI\PaymentType();
        $payment->setCreditCard($creditCard);
        $subscription->setPayment($payment);


        $order = new AnetAPI\OrderType();
        $order->setInvoiceNumber($invoiceNumber);
        $order->setDescription("Subscription");
        $subscription->setOrder($order);

        $billTo = new AnetAPI\NameAndAddressType();
        $billTo->setFirstName(\Auth::guard('api')->user()->first_name);
        $billTo->setLastName(\Auth::guard('api')->user()->last_name);

        $subscription->setBillTo($billTo);

        $request = new AnetAPI\ARBCreateSubscriptionRequest();
        $request->setmerchantAuthentication($merchantAuthentication);
        $request->setRefId($refId);
        $request->setSubscription($subscription);
        $controller = new AnetController\ARBCreateSubscriptionController($request);

        // Create the controller and get the response
        if (env('PAYMENT_ENVIRONMENT') == 'SANDBOX') {
           
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::SANDBOX);

        } else {
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::PRODUCTION);
        }

        if (($response != null) && ($response->getMessages()->getResultCode() == "Ok")) {

            $update = [
                'subscription_id' => $subscriptionId,
                'subscription_type' => $subscriptionType,
                'authorize_subscription_id' => $response->getSubscriptionId(),
                'authorize_subscription_status' => 1,
                'subscription_start_date' => date('Y-m-d H:i:s'),
                'subscription_end_date' => date('Y-m-d H:i:s', strtotime('+100 years')),
            ];

            \App\Models\AppUser::where('app_user_id', $userId)->update($update);

            $subscription = \App\Models\Subscription::where('subscription_id', $subscriptionId)->pluck('subscription_name');

            $paymentLog = [
                [
                    'app_user_id' => \Auth::guard('api')->user()->app_user_id,
                    'authorize_net_id' => $response->getSubscriptionId(),
                    'type' => 1,
                    'message_code' => 'net.authorize.customer.subscription.created',
                    'message' => 'Subscription Successfully',
                    'amount' => $amount,
                    'subscription_id' => $subscriptionId,
                    'subscription_type' => $subscriptionType,
                    'date' => date('Y-m-d H:i:s'),
                    'invoice_number' => $invoiceNumber,
                    'status' => 'Create'
                ], [
                    'app_user_id' => \Auth::guard('api')->user()->app_user_id,
                    'authorize_net_id' => '',
                    'type' => 2,
                    'message_code' => '',
                    'message' => 'Transaction Successfully',
                    'amount' => $amount,
                    'subscription_id' => $subscriptionId,
                    'subscription_type' => $subscriptionType,
                    'date' => date('Y-m-d H:i:s'),
                    'invoice_number' => $invoiceNumber,
                    'status' => 'Success'
                ]

            ];

            \App\Models\PaymentLog::insert($paymentLog);
            $msg = $response->getMessages()->getMessage()[0]->getText();

            $data = [
                'start' => $startDate,
                'name' =>  \Auth::guard('api')->user()->first_name . ' ' . \Auth::guard('api')->user()->last_name,
                'subscripation' => $subscription[0],
                'email' => \Auth::guard('api')->user()->email
            ];

            try {
                Mail::send('mails.subscription-successfully', ['data' => $data], function ($message) use ($data) {
                    $message->to($data['email'], $data['name'])->subject('Luxury Council –Subscription Purchased successfully');
                });
            } catch (\Exception $e) {
                Log::error('This is an error message.' . $e);
            }


            $detail = [
                'amount' => $amount,
                'status' => true,
                'transaction_id' => $response->getSubscriptionId(),
                'auth_id' => '',
                'message_code' => '',
                'message' =>   $response->getMessages()->getMessage()[0]->getText(),
                'created_at' => date('Y-m-d H:i:s'),
                'app_user_id' => $userId,
                'subscription_id' => $subscriptionId,
                'subscription_type' => $subscriptionType,
            ];

            $response = [
                'data' => $detail,
                'status' => true,
                'message' => $msg
            ];
        } else {
            $response = [
                'data' => (object) [],
                'status' => false,
                'message' =>   $response->getMessages()->getMessage()[0]->getText()
            ];
        }

        return response()->json($response);
    }

    /**
     * @OA\Get(
     *     path="/api/transaction-list",
     *     operationId="transactionList",
     *     summary="Transaction List",
     *     description="Transaction List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\Response(
     *         response=200,
     *         description="Transaction List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function transactionList(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $offset = $this->offset($page, $limit);

            $userId = \Auth::guard('api')->user()->app_user_id;
            $paymentLog = PaymentLog::select('payment_log_id', 'transaction_id', 'auth_id', 'message_code', 'message', 'amount')->where('app_user_id',  $userId)->orderBy('payment_log_id', 'DESC')->skip($offset)->take($limit)->get()->toArray();

            if (!empty($paymentLog)) {
                $response = [
                    'data' => $paymentLog,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Payment Log'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/transaction-detail",
     *     operationId="transactionDetail",
     *     summary="Transaction Detail",
     *     description="Transaction Detail",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"transaction_id"},
     *                 @OA\Property(property="transaction_id", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Transaction Detail",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function transactionDetail(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                'transaction_id' => [
                    'required',
                    Rule::exists('payment_logs')->where(function ($query) {
                        $query->whereNull('deleted_at');
                    }),
                ],
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $transactionId = $request->get('transaction_id');

            $event = PaymentLog::select('payment_log_id', 'transaction_id', 'auth_id', 'message_code', 'message', 'amount')
                ->where('transaction_id', $transactionId)->first()->toArray();


            if (!empty($event)) {
                $response = [
                    'data' => $event,
                    'status' => true,
                    'message' => __('messages.detail', ['name' => 'Transaction'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }


            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Get(
     *     path="/api/cancel-subscription",
     *     operationId="cancelSubscription",
     *     summary="Cancel Subscription",
     *     description="Cancel Subscription",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\Response(
     *         response=200,
     *         description="Cancel Subscription",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function cancelSubscription(Request $request)
    {

        try {
            $authorizeSubscriptionId = \Auth::guard('api')->user()->authorize_subscription_id;
            $userId = \Auth::guard('api')->user()->app_user_id;


            /* Create a merchantAuthenticationType object with authentication details
                retrieved from the constants file */
            $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
            $merchantAuthentication->setName(env('MERCHANT_LOGIN_ID'));
            $merchantAuthentication->setTransactionKey(env('MERCHANT_TRANSACTION_KEY'));

            // Set the transaction's refId
            $refId = 'ref' . time();

            $request = new AnetAPI\ARBCancelSubscriptionRequest();
            $request->setMerchantAuthentication($merchantAuthentication);
            $request->setRefId($refId);
            $request->setSubscriptionId($authorizeSubscriptionId);

            $controller = new AnetController\ARBCancelSubscriptionController($request);

            if (env('PAYMENT_ENVIRONMENT') == 'SANDBOX') {
                $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::SANDBOX);
            } else {
                $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::PRODUCTION);
            }

            if (($response != null) && ($response->getMessages()->getResultCode() == "Ok")) {

                \App\Models\AppUser::where('app_user_id', $userId)->update([
                    'authorize_subscription_status' => 0,
                    'subscription_end_date' => date('Y-m-d H:i:s')
                ]);

                $subscription = \App\Models\Subscription::where('subscription_id', \Auth::guard('api')->user()->subscription_id)->first();
                if(\Auth::guard('api')->user()->subscription_type == 1){
                    $amount = $subscription->subscription_price_per_month;
                } else {
                    $amount = $subscription->subscription_price_per_year;
                }

                $paymentLog = [
                    'app_user_id' => \Auth::guard('api')->user()->app_user_id,
                    'authorize_net_id' => $authorizeSubscriptionId,
                    'type' => 1,
                    'message_code' => 'net.authorize.customer.subscription.cancelled',
                    'message' => 'Subscription Cancelled',
                    'amount' => $amount,
                    'subscription_id' => \Auth::guard('api')->user()->subscription_id,
                    'subscription_type' => \Auth::guard('api')->user()->subscription_type,
                    'date' => date('Y-m-d H:i:s'),
                    'status' => 'Cancel'
                ];

                \App\Models\PaymentLog::insert($paymentLog);

                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.cancel_subscription')
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>   "No response returned"
                ];
            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }
}
