<?php

namespace App\Http\Controllers;

use Mail;
use DataTables;
use Carbon\Carbon;
use App\Models\AppUser;
use App\Models\Company;
use App\Models\Country;
use App\Models\PaymentLog;
use App\Models\Subscription;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use App\Models\GroupSubscription;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

class AppUserController extends Controller
{
    public function __construct()
    {
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $group_subscription_id = $request->query('group_subscription_id') ? $request->query('group_subscription_id') : '';
        return view('app-user.index', ['group_subscription_id' => $group_subscription_id]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $company = Company::orderBy("company_name", "asc")->pluck('company_name', 'company_id');
        $subscripation = Subscription::pluck('subscription_name', 'subscription_id');
        $interest = \App\Models\Interest::orderBy('interest_name', 'ASC')->pluck('interest_name', 'interest_id');
        $country = Country::select(DB::raw('CONCAT(short_name, " ", "(+", code, ")") AS short_code'), 'short_name')->pluck('short_code', 'short_name');
        return view('app-user.form', ['subscriptions' => $subscripation, 'company' => $company, 'interest' => $interest, 'country' => $country]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = $request->all();

        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => ['required', 'email', Rule::unique('app_users')->whereNull('deleted_at')],
            'password' => 'min:8|required_with:confim_password|same:confim_password|regex:/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,}$/',
            'confim_password' => 'min:8|required',
            'company' => 'required',
            'title' => 'required',
            'phone_no' => ['required', 'numeric', 'digits:10', Rule::unique('app_users')->whereNull('deleted_at')],
            'mailing_address' => 'required',
            'assistant_email' => 'nullable|email',
            'assistant_phone_no' => 'nullable|numeric|digits:10',
            'app_user_status' => 'required',
        ];

        $messages = [
            'password.regex' => 'Password must have at least one uppercase letter, one lowercase letter, one digit, and one special character.',
        ];

        $request->validate($rules, $messages);

        $image_name = '';
        if ($request->hasFile('profile_img')) {

            $file = $request->file('profile_img');
            $image_name = time() . '.' . $file->getClientOriginalExtension();

            $directory = public_path('/profile_image/');

            if (!is_dir($directory)) {
                mkdir($directory, 0777, true);
        }

            $file->move($directory , $image_name);
        }


        $startDate = $endDate = null;
        if (isset($data['subscription_type'])) {
            $type =  isset($data['subscription_type']) && $data['subscription_type'] == 1 ? '+1 months' : '+1 year';
            $startDate = date('Y-m-d', strtotime($data['subscription_start_date'])) . ' ' . date('H:i:s');
            $endDate = date('Y-m-d H:i:s', strtotime($type  . ' ' . $startDate));
        }

        $appUserData = [
            "first_name" => $data['first_name'],
            "last_name" => $data['last_name'],
            "email" =>  $data['email'],
            "password" =>  Hash::make($data['password']),
            "company" =>  $data['company'],
            "title" =>  $data['title'],
            "country_code" =>  isset($data['country_code']) ? $data['country_code'] : 'US',
            "phone_no" =>  $data['phone_no'],
            "mailing_address" =>  isset($data['mailing_address']) ? $data['mailing_address'] : '',
            "assistant_name" =>   isset($data['assistant_name']) ? $data['assistant_name'] : '',
            "assistant_email" =>  isset($data['assistant_email']) ? $data['assistant_email'] : '',
            "assistant_country_code" =>  isset($data['assistant_country_code']) ? $data['assistant_country_code'] : 'US',
            "assistant_phone_no" =>   isset($data['assistant_phone_no']) ? $data['assistant_phone_no'] : '',
            "subscription_id" =>   isset($data['subscription_id']) ? $data['subscription_id'] : 0,
            "subscription_type" =>  isset($data['subscription_type']) ? $data['subscription_type'] : 0,
            "app_user_status" =>  $data['app_user_status'],
            "subscription_start_date" =>   $startDate,
            "subscription_end_date" =>   $endDate,
            "birthday" =>  isset($data['birthday']) ? $data['birthday'] : null,
            "business_interests" =>  isset($data['business_interests']) ? $data['business_interests'] : '',
            "spouse_name" =>   isset($data['spouse_name']) ? $data['spouse_name'] : '',
            "spouse_birthday" =>  isset($data['spouse_birthday']) ? $data['spouse_birthday'] : null,
            "profile_img" =>  $image_name,
        ];

        $user = AppUser::create($appUserData);

        if (isset($data['child_name'])) {
            $children = [];
            foreach ($data['child_name'] as $key => $val) {
                if (!is_null($val) &&  !is_null($data['child_birthday'][$key])) {
                    $children[] = [
                        'name' => $val,
                        'birthday' => $data['child_birthday'][$key],
                        'app_user_id' => $user->app_user_id
                    ];
                }
            }
            if (!empty($children)) {
                \App\Models\AppUserChildDetail::insert($children);
            }
        }

        Mail::send('mails.credential', ['data' => $data], function ($message) use ($data) {
            $message->to($data['email'], $data['first_name'] . ' ' . $data['last_name'])->subject('Luxury Council - App Credential');
        });

        return redirect()->route('app.user.index')->with('message',  __('messages.add', ['name' => 'App User']));
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $subscripation = Subscription::pluck('subscription_name', 'subscription_id');
        $company = Company::orderBy("company_name", "asc")->pluck('company_name', 'company_id');
        $appUser = AppUser::where('app_user_id', $id)->first();
        $childern = \App\Models\AppUserChildDetail::where('app_user_id', $id)->get()->toArray();
        $interest = \App\Models\Interest::orderBy('interest_name', 'ASC')->pluck('interest_name', 'interest_id');

        $groupName = '';
        if ($appUser->group_subscription_id != 0 || $appUser->group_subscription_id != null) {
            $groupSub = GroupSubscription::select('company_name')->where('group_subscription_id', $appUser->group_subscription_id)->first();
            $companyDetail = Company::select('company_name')->where('company_id', $groupSub->company_name)->first();
            $groupName = $companyDetail->company_name;
        }

        $country = Country::select(DB::raw('CONCAT(short_name, " ", "(+", code, ")") AS short_code'), 'short_name')->pluck('short_code', 'short_name');
        return view('app-user.form', ['appUser' => $appUser, 'subscriptions' => $subscripation, 'groupName' => $groupName, 'company' => $company, 'childern' => $childern, 'interest' => $interest, 'country' => $country]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = $request->all();
        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => ['required', 'email', Rule::unique('app_users')->whereNull('deleted_at')->whereNot('app_user_id', $id)],
            'password' => 'required_with:confim_password|same:confim_password',
            'confim_password' => 'required_with:password',
            'company' => 'required',
            'title' => 'required',
            'phone_no' => ['required', 'numeric', 'digits:10', Rule::unique('app_users')->whereNull('deleted_at')->whereNot('app_user_id', $id)],
            'mailing_address' => 'required',
            'assistant_email' => 'nullable|email',
            'assistant_phone_no' => 'nullable|numeric|digits:10',
            'app_user_status' => 'required',
            'birthday' => 'nullable|not_in:0000-00-00',
            'spouse_birthday' => 'nullable|not_in:0000-00-00',
        ];
        $request->validate($rules);

        $startDate = $endDate = null;
        if (isset($data['subscription_type'])) {
            $type =  isset($data['subscription_type']) && $data['subscription_type'] == 1 ? '+1 months' : '+1 year';
            $startDate = date('Y-m-d', strtotime($data['subscription_start_date'])) . ' ' . date('H:i:s');
            $endDate = date('Y-m-d H:i:s', strtotime($type  . ' ' . $startDate));
        }



        $appUserData = [
            "first_name" => $data['first_name'],
            "last_name" => $data['last_name'],
            "email" =>  $data['email'],
            "company" =>  $data['company'],
            "title" =>  $data['title'],
            "country_code" =>  isset($data['country_code']) ? $data['country_code'] : 'US',
            "phone_no" =>  $data['phone_no'],
            "mailing_address" =>  $data['mailing_address'],
            "assistant_name" =>  $data['assistant_name'],
            "assistant_email" =>  $data['assistant_email'],
            "assistant_country_code" =>  isset($data['assistant_country_code']) ? $data['assistant_country_code'] : 'US',
            "assistant_phone_no" =>  $data['assistant_phone_no'],
            "subscription_id" =>  $data['subscription_id'],
            "app_user_status" =>  $data['app_user_status'],
            "subscription_start_date" =>  $startDate,
            "subscription_end_date" =>  $endDate,
            "birthday" =>  $data['birthday'],
            "business_interests" =>  !empty($data['business_interests']) ? implode(',', $data['business_interests']) : '',
            "spouse_name" =>  $data['spouse_name'],
            "spouse_birthday" =>  $data['spouse_birthday'],
        ];

        $base64Image = '';
        if ($request->hasFile('profile_img')) {

            $file = $request->file('profile_img');
            $image_name = time() . '.' . $file->getClientOriginalExtension();

            $directory = public_path('/profile_image/');

            if (!is_dir($directory)) {
                mkdir($directory, 0777, true);
            }

            $file->move($directory , $image_name);

            $appUserData['profile_img']  = $image_name;
            if (AppUser::where('app_user_id', $id)->first()->profile_img != '' && file_exists($directory. AppUser::where('app_user_id', $id)->first()->profile_img)) {
                unlink(public_path('/profile_image/') . AppUser::where('app_user_id', $id)->first()->profile_img);
            }
        }

        if ($data['remove_profile_flag'] == 1) {
            $appUserData['profile_img'] = '';
        }

        if (isset($data['subscription_type'])) {
            $appUserData['subscription_type'] = $data['subscription_type'];
        }
        if (!empty($data['password'])) {
            $appUserData['password'] = Hash::make($data['password']);
        }
        AppUser::where('app_user_id', $id)->update($appUserData);

        \App\Models\AppUserChildDetail::where('app_user_id', $id)->delete();
        if (isset($data['child_name'])) {
            $children = [];
            foreach ($data['child_name'] as $key => $val) {
                if ($val != null && $data['child_birthday'][$key] != null) {
                    $children[] = [
                        'name' => $val,
                        'birthday' => $data['child_birthday'][$key],
                        'app_user_id' =>  $id
                    ];
                }
            }
            if (!empty($children)) {
                \App\Models\AppUserChildDetail::insert($children);
            }
        }

        if(isset(Auth::guard('app_user')->user()->app_user_id)){
            
            return redirect()->route('transaction.user.list')->with('message',  __('messages.edit', ['name' => 'App User']));
        }else{
            return redirect()->route('app.user.index')->with('message',  __('messages.edit', ['name' => 'App User']));
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        AppUser::where('app_user_id', $id)->update(['deleted_at' => Carbon::now()]);
        PaymentLog::where('app_user_id', $id)->update(['deleted_at' => Carbon::now()]);
        return redirect()->route('app.user.index')->with('message',  __('messages.delete', ['name' => 'App User']));
    }

    public function ajaxList(Request $request)
    {

        if ($request->ajax()) {
            $columns = ['action','first_name','title', 'email', 'phone_no', 'company.company_name', 'subscription_id','subscription_type','subscription_start_date', 'is_group_subscription', 'app_user_status'];

            $data_spotlight_member = AppUser::select('app_user_id', 'first_name', 'last_name', 'email', 'company.company_name', 'title', 'phone_no', 'subscription_id','subscription_type', 'app_user_status', 'group_subscription_id', 'subscription_start_date');
            $data_spotlight_member->leftJoin('company', 'company.company_id', '=', 'app_users.company');

            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_spotlight_member->orderBy($columnName, $columnSortOrder);

            if($request->group_subscription_id != 0){
                $data_spotlight_member->where('group_subscription_id', $request->group_subscription_id);
            }
            $data = $data_spotlight_member->get();
            return DataTables::of($data_spotlight_member)
                 ->addColumn('app_user_status', function ($row) {
                    return $row->app_user_status == 1 ? "Active" : "Inactive";
                })->addColumn('full_name', function ($row) {
                    return $row->first_name . ' ' . $row->last_name;
                })
                ->addColumn('is_group_subscription', function ($row) {
                    return $row->group_subscription_id != 0 ? "Yes" : "No";
                })
                ->addColumn('subscription_id', function ($row) {
                    if($row->subscription_id != 0 || $row->subscription_id != ''){
                    $subscription = \App\Models\Subscription::select('subscription_name')->where('subscription_id', $row->subscription_id)->first();
                        return !is_null($subscription) ?  $subscription->subscription_name : '';
                    }
                    return '';

                })
                ->addColumn('company', function ($row) {
                    return $row->company_name ? $row->company_name : '';
                })
                ->addColumn('subscription_type', function ($row) {
                    return $row->subscription_type == 1 ? 'Monthly' : ($row->subscription_type == 2  ? 'Yearly' : '-');
                })
                ->addColumn('action', function ($row) {
                    $editUrl = route('app.user.edit', $row->app_user_id);
                    $deleteUrl = route('app.user.destroy', $row->app_user_id);
                    $transaction_list = '<a href="' . route('transaction.index', ['app_user_id' =>  $row->app_user_id]) . '" class="btn"  style="background: #e2e6ea;" title="Transaction"><i class="fa fa-money" aria-hidden="true"></i></a>';

                    return '<a href="' . $editUrl . '" class="btn" style="background: #000000;color: #f0ac63 !important;"  title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>

                    <form action="'.route('app.user.destroy', $row->app_user_id).'" method="post" style="display:inline;" id="delete_'. $row->app_user_id .'">
                        <button class="btn" type="button" value="Delete" style="background: #f0ac63;color: #000000 !important;"   onclick="return deleteRecord($(this));" data-url="'. $row->app_user_id  .'" title="Delete"><i class="fa fa-trash" aria-hidden="true"></i></button>
                        <input type="hidden" name="_method" value="delete"/>
                        <input type="hidden" name="_token" value="'. csrf_token().'">
                    </form> ' . $transaction_list;
                })
                ->filter(function ($query) use ($request) {

                    if (isset($request->full_name)) {
                        $search = '%' . $request->full_name . '%';
                        $query->whereRaw("CONCAT(first_name, ' ', last_name) LIKE  '$search'");
                    }
                    if (isset($request->spotlight_member_long_desc)) {
                        $search = '%' . $request->spotlight_member_long_desc . '%';
                        $query->where("spotlight_member_long_desc", "LIKE", $search);
                    }
                    if (isset($request->email)) {
                        $search = '%' . $request->email . '%';
                        $query->where("email", "LIKE", $search);
                    }
                    if (isset($request->company)) {
                        $search = '%' . $request->company . '%';
                        $query->where("company.company_name", "LIKE", $search);
                    }
                    if (isset($request->title)) {
                        $search = '%' . $request->title . '%';
                        $query->where("title", "LIKE", $search);
                    }
                    if (isset($request->phone_no)) {
                        $search = '%' . $request->phone_no . '%';
                        $query->where("phone_no", "LIKE", $search);
                    }
                })
                ->rawColumns(['action', 'app_user_status', 'subscription_id', 'subscription_type', 'full_name', 'company'])
                ->addIndexColumn()
                ->make(true);
        }
    }


    // public function userTransactionList(){
    //     dd('trasection-list');
    // }
}
